

import os
import re

import json
import glob

import subprocess

from deepsearch_glm.utils.load_pretrained_models import load_pretrained_nlp_models

ROOT_DIR=os.path.abspath("./")
BUILD_DIR=os.path.join(ROOT_DIR, "build")

def run(cmd, cwd="./"):

    print(f"\nlaunch: {cmd}")
    
    parts = cmd.split(" ")
    message = subprocess.run(parts, cwd=cwd)    

    if "returncode=0" in str(message):
        print(f" -> SUCCESS")
        return True

    print(f" -> ERROR with message: '{message}'\n")        
    return False
    
def build(setup_kwargs=None):

    if not os.path.exists(BUILD_DIR):
        cmd = f"cmake -B {BUILD_DIR}"
        run(cmd, cwd=ROOT_DIR)
        
    cmd = f"cmake --build {BUILD_DIR} --target install -j"
    run(cmd, cwd=ROOT_DIR)    

def build_all_python_versions():

    candidates = glob.glob("/usr/local/bin/python3.*")
    candidates += glob.glob("/usr/bin/python3.*")

    print(f"all candidates: {candidates}")
    
    python_versions=[]
    for candidate in candidates:
        pyname = os.path.basename(candidate)
        if re.match("^python3.\d+$", pyname):
            python_versions.append(candidate)

    python_versions = sorted(python_versions)
    print(f"all found python-versions: {python_versions}")
    
    for pyv in python_versions:

        pyn = os.path.basename(pyv)        
        PYBUILD_DIR = os.path.join(ROOT_DIR, f"build-{pyn}")
        if os.path.exists(PYBUILD_DIR):
            print(f"rm {PYBUILD_DIR}")
            continue
        
        cmd = f"cmake -B {PYBUILD_DIR} -DPYTHON_EXECUTABLE={pyv}"
        run(cmd, cwd=ROOT_DIR)
            
        cmd = f"cmake --build {PYBUILD_DIR} --target install -j"
        run(cmd, cwd=ROOT_DIR)    
    
if "__main__"==__name__:

    #load_pretrained_nlp_models(False)

    #build()
    build_all_python_versions()






#!/usr/bin/env python

import os
import json
import glob
import argparse

import pandas as pd

from utils.ds_utils import convert_pdffiles

import andromeda_nlp

def parse_arguments():

    parser = argparse.ArgumentParser(
        prog = 'apply_nlp_on_doc',
        description = 'Apply NLP on `Deep Search` documents',
        epilog =
"""
examples of execution: 

1.a run on single document (pdf or json) with default model (=`langauge`):

     poetry run python ./deepsearch_glm/apply_nlp_on_docs.py --pdf './data/documents/articles/2305.02334.pdf'
     poetry run python ./deepsearch_glm/apply_nlp_on_docs.py --json './data/documents/articles/2305.02334.json'

1.b run on single document pdf document and enforce conversion (ignore cache):

     poetry run python ./deepsearch_glm/apply_nlp_on_docs.py --pdf './data/documents/articles/2305.02334.pdf' --force-convert True

2. run on multiple documents:

     poetry run python ./deepsearch_glm/apply_nlp_on_docs.py --pdf './data/documents/articles/*.pdf'
     poetry run python ./deepsearch_glm/apply_nlp_on_docs.py --json './data/documents/articles/*.json'

3. run on multiple documents with non-default models:

     poetry run python ./deepsearch_glm/apply_nlp_on_docs.py --pdf './data/documents/articles/2305.*.pdf' --models 'language;term'

""",
        formatter_class=argparse.RawTextHelpFormatter)

    parser.add_argument('--pdf', required=False,
                        type=str, default=None,
                        help="filename(s) of pdf document")
    
    parser.add_argument('--json', required=False,
                        type=str, default=None,
                        help="filename(s) of json document")

    parser.add_argument('--models', required=False,                        
                        type=str, default="language",
                        help="set NLP models (e.g. `language` or `term;sentence`)")

    parser.add_argument('--force-convert', required=False,
                        type=bool, default=False,
                        help="force pdf conversion")
    
    args = parser.parse_args()

    pdf = args.pdf
    json = args.json

    if pdf==None and json==None:
        exit(-1)
        
    if pdf!=None:
        pdf_files=sorted(glob.glob(pdf))
    else:
        pdf_files=[]
        
    if json!=None:
        json_files=sorted(glob.glob(json))
    else:
        json_files=[]
        
    return pdf_files, json_files, args.models, args.force_convert

def init_nlp_model(models:str):
    
    model = andromeda_nlp.nlp_model()

    config = model.get_apply_configs()[0]
    config["models"] = models
    
    model.initialise(config)

    return model

if __name__ == '__main__':

    pdf_files, json_files, model_names, force_convert = parse_arguments()

    if len(pdf_files)>0:
        new_json_files = convert_pdffiles(pdf_files, force=force_convert)

        for _ in new_json_files:
            json_files.append(_)

    json_files = sorted(list(set(json_files)))        

    model = init_nlp_model(model_names)

    for json_file in json_files:

        with open(json_file, "r") as fr:
            doc_i = json.load(fr)
    
        doc_j = model.apply_on_doc(doc_i)

        props = pd.DataFrame(doc_j["properties"]["data"],
                             columns=doc_j["properties"]["headers"])
        print("properties: \n\n", props)

        ints = pd.DataFrame(doc_j["instances"]["data"], 
                            columns=doc_j["instances"]["headers"])
        print("instances: \n\n", ints)

        """
        ents = pd.DataFrame(doc_j["entities"]["data"], 
                            columns=doc_j["entities"]["headers"])
        print(ents)
        """
        
        nlp_file = json_file.replace(".json", ".nlp.json")
        with open(nlp_file, "w") as fw:
            fw.write(json.dumps(doc_j, indent=2))


    




import os

import json
import glob

import argparse
import textwrap
import datetime
import subprocess

#from tabulate import tabulate

from utils.ds_utils import get_scratch_dir

import andromeda_nlp

def load_models():

    if "DEEPSEARCH_GLM_RESOURCES_DIR" in os.environ:
        RESOURCES_DIR = os.getenv("DEEPSEARCH_GLM_RESOURCES_DIR")
    else:
        # FIXME: not sure about this ...
        ROOT_DIR=os.path.abspath("./")
        RESOURCES_DIR=os.path.join(ROOT_DIR, "resources")
    
    with open(f"{RESOURCES_DIR}/models.json") as fr:
        models = json.load(fr)
        
    COS_URL = models["object-store"]

    cmds=[]
    for name,files in models["trained-models"].items():
        source = os.path.join(COS_URL, files[0])
        target = os.path.join(RESOURCES_DIR, files[1])
        
        cmds.append(["curl", source, "-o", target, "-s"])
        #print(" ".join(cmds[-1]))
        
    for cmd in cmds:
        if not os.path.exists(cmd[3]):
            print(f"downloading {os.path.basename(cmd[3])} ... ", end="")
            message = subprocess.run(cmd, cwd=ROOT_DIR)    
            print("done!")
        else:
            print(f" -> already downloaded {os.path.basename(cmd[3])}")
    

def create_nlp_dir():

    tdir = get_scratch_dir()

    now = datetime.datetime.now()
    nlpdir = now.strftime("NLP-model-%Y-%m-%d_%H-%M-%S")

    odir = os.path.join(tdir, nlpdir)
    return odir

def list_nlp_model_configs():

    configs = []

    configs += nlp_model.get_apply_configs()
    configs += nlp_model.get_train_configs()

    return configs

def init_nlp_model(model_names:str="language;term"):

    nlp_model = andromeda_nlp.nlp_model()

    config = nlp_model.get_apply_configs()[0]
    config["models"] = model_names

    nlp_model.initialise(config)
    
    return nlp_model



#!/usr/bin/env python

import os

import json
import glob

import argparse
import textwrap

from tabulate import tabulate

from ds_convert import convert_pdffile

import andromeda_nlp
import andromeda_glm

def parse_arguments():

    parser = argparse.ArgumentParser(
        prog = 'glm_docqa',
        description = 'Do Q&A on pdf document',
        epilog = 'Text at the bottom of help')

    parser.add_argument('--pdf', required=True,
                        type=str,
                        help="filename of pdf document")

    parser.add_argument('--force', required=False, 
                        type=bool, default=False,
                        help="force pdf conversion")
    
    parser.add_argument('--models', required=False,                        
                        type=str, default="name;verb;term;abbreviation",
                        help="set NLP models (e.g. `term;sentence`)")
    
    args = parser.parse_args()

    return args.pdf, args.force, args.models
    
def load_nlp(models:str="name;conn;verb;term;language;reference;abbreviation"):

    nlp_model = andromeda_nlp.nlp_model()

    config = nlp_model.get_apply_configs()[0]
    config["models"] = models
    
    nlp_model.initialise(config)    

    return nlp_model

def apply_nlp(doc_i):

    doc_j = model.apply_on_doc(doc_i)

def load_glm(path:str):

    config = {
        "IO": {
	    "load": {
                "root": path
            }
        }
    }
    
    glm_model = andromeda_glm.glm_model()
    glm_model.load(config)

    return glm_model

def analyse_prompt(prompt, nlp_model):

    res = nlp_model.apply_on_text(prompt)

    print(res.keys())
    print(tabulate(res["word-tokens"]["data"], headers=res["word-tokens"]["headers"]))
    print(tabulate(res["entities"]["data"], headers=res["entities"]["headers"]))

    type_ind = res["entities"]["headers"].index("type")
    name_ind = res["entities"]["headers"].index("name")
    
    terms=[]
    for i,row in enumerate(res["entities"]["data"]):
        if(row[type_ind]=="term"):
            terms.append([row[name_ind]])

    return terms

def show_query_result(res, max_nodes=16):

    wrapper = textwrap.TextWrapper(width=50)
    
    print("overview: \n", tabulate(res["overview"]["data"],
                                   headers=res["overview"]["headers"]), "\n")

    for i,item in enumerate(res["result"]):
        
        headers = item["nodes"]["headers"]
        data = item["nodes"]["data"]

        for j,row in enumerate(data):
            text = row[headers.index("text")]
            print("text: ", text)
            
            data[j][headers.index("text")] = "\n".join(wrapper.wrap(text))
                    
        print(f"operation {i}: \n", tabulate(data[0:max_nodes], headers=headers), "\n")

def expand_terms(terms):

    for term in terms:
        print(term)
        
        qry = andromeda_glm.glm_query()
        qry.select({"nodes":[[term]]})
        qry.filter_by({"mode": "node-flavor", "node-flavors":["token"]})
        flid = qry.get_last_flid()
        qry.traverse({"edge":"to-root"})
        qry.traverse({"edge":"from-root"})
        qry.filter_by({"mode": "node-flavor", "node-flavors":["term"]})

        qry.filter_by({"mode": "contains", "contains-flid":flid})        
        qry.traverse({"edge":"to-sent"})
        
        config = qry.to_config()    
        #print("query: ", json.dumps(config, indent=2))    
        
        res = glm_model.query(config)
        if "status" in res and res["status"]=="success":
            show_query_result(res)
        else:
            print(res)
            #print(res["status"], ": ", res["message"])
            
def do_qa(nlp_model, glm_model):

    while True:
        
        prompt = input("question: ")
        #prompt = "What is the income of IBM in 2022?"
        #prompt = "net-zero"
        if(prompt=="q"):
            break
        
        #terms = analyse_prompt(prompt, nlp_model)
        terms = prompt.split(" ")
        
        expand_terms(terms)
        
        """
        qry = andromeda_glm.glm_query()
        qry.select({"nodes":search_terms})
        qry.filter_by({"mode": "node-flavor", "node-flavors":["term"]})
        qry.traverse({"edge":"to-table"})
        
        config = qry.to_config()    
        print("query: ", json.dumps(config, indent=2))    

        res = glm_model.query(config)
        show_query_result(res)
        """

        #break
        
if __name__ == '__main__':

    pdffile, force, models = parse_arguments()

    success, jsonfile = convert_pdffile(pdffile, force=force)

    if not success:
        return -1
    
    print(f"json-file: {jsonfile}")
    nlp_model = load_nlp(models)    

    with open(jsonfile, "r") as fr:        
        doc_i = json.load(fr)
        doc_j = nlp_model.apply_on_doc(doc_i)

    #for item in doc_j["main-text"]:
    #    print(item)
    
    #glm_model = load_glm("../build/glm-model-reports")
    #do_qa(nlp_model, glm_model)



#!/usr/bin/env python

import os
import json
import glob

import argparse
import textwrap

import pandas as pd

import textColor as tc

import andromeda_nlp

default_text = """A team of physicists at Université Paris-Saclay has, for the first time, observed spontaneous quasi-crystal self-assembly. The observation occurred during an experiment they were conducting with tiny vibrating magnetic spheres. The team has written a paper describing their experiment and have posted it on the arXiv preprint server while they await peer review."""

def parse_arguments():

    parser = argparse.ArgumentParser(
        prog = 'nlp_on_text',
        description = 'Apply NLP on text',
        epilog =
"""
examples of execution:

1.a run on single document (pdf or json) with default model (=`langauge`):

    poetry run python ./deepsearch_glm/nlp_analyse_docs.py --json ./data/documents/articles/2305.02334.nlp.json

""",
        formatter_class=argparse.RawTextHelpFormatter)

    parser.add_argument('-t', '--text', required=False,
                        type=str, default=default_text,
                        help="text on which the NLP is applied.")

    parser.add_argument('-m', '--model-names', required=False,
                        type=str, default="verb;term;conn;semantic;abbreviation",
                        help="model_names to be applied")

    
    args = parser.parse_args()
    
    return args.text, args.model_names

if __name__ == '__main__':

    text, model_names = parse_arguments()
    
    mdl = andromeda_nlp.nlp_model()
    print("resource-dir: ", mdl.get_resources_path())

    mdl.initialise_models(model_names)

    result = mdl.apply_on_text(text)

    props = pd.DataFrame(result["properties"]["data"],
                         columns=result["properties"]["headers"])
    print(tc.yellow("properties: \n\n"), props, "\n")

    ints = pd.DataFrame(result["instances"]["data"], 
                        columns=result["instances"]["headers"])
    print(tc.yellow("instances: \n\n"), ints, "\n")
    
    """
    ents = pd.DataFrame(result["entities"]["data"], 
    columns=result["entities"]["headers"])
    print(ents)
    """



#!/usr/bin/env python

import os
import json
import glob

import argparse
import textwrap

import pandas as pd

from tabulate import tabulate
#from ds_utils import convert_pdffiles

import andromeda_nlp

def parse_arguments():

    parser = argparse.ArgumentParser(
        prog = 'nlp_analyse_docs',
        description = 'Analyse NLP on `Deep Search` documents ()',
        epilog =
"""
examples of execution:

1.a run on single document (pdf or json) with default model (=`langauge`):

    poetry run python ./deepsearch_glm/nlp_analyse_docs.py --json ./data/documents/articles/2305.02334.nlp.json

""",
        formatter_class=argparse.RawTextHelpFormatter)

    parser.add_argument('--json', required=True,
                        type=str, default=None,
                        help="filename(s) of json document")

    args = parser.parse_args()
    
    json_files=sorted(glob.glob(args.json))

    return json_files

def extract_meta(doc):
    return

def extract_text(doc):

    page_items = doc["page-items"]
    texts = doc["texts"]
    
    wrapper = wrapper = textwrap.TextWrapper(width=70)
    
    for item in doc["texts"]:

        """
        rows=[]
        for prov in item["prov"]:
            rows.append([prov["page"], prov["type"], int(prov["bbox"][0]), int(prov["bbox"][1]), int(prov["bbox"][2]), int(prov["bbox"][3])] )

        headers=["page", "type", "x0", "y0", "x1", "y1"]
        print("provenance: \n", tabulate(rows, headers=headers))
        """

        labels=[]
        for row in item["properties"]["data"]:
            labels.append(row[item["properties"]["headers"].index("label")])
        
        type_ = item["prov"][0]["type"]
        print(f"text: {type_}, labels: ", ",".join(labels))
        for line in wrapper.wrap(text=item["text"]):
            print(f"\t{line}")

        print("")
    return

def extract_sentences(doc):

    """
    for row in doc["instances"]:
        print(len(row), "\t", row)
    """

    df = pd.DataFrame(doc["instances"]["data"],
                      columns=doc["instances"]["headers"])
    #print(df)

    entity_types = df["type"].value_counts()
    print(entity_types)
    
    sents = df[df["type"]=="sentence"]
    print(sents)
    
    return

def extract_tables(doc):

    wrapper = wrapper = textwrap.TextWrapper(width=70)
    
    for i,item in enumerate(doc["tables"]):

        print(f"#/tables/{i}: ", len(item["captions"]))
        if len(item["captions"])>0:
            for line in wrapper.wrap(text=item["captions"][0]["text"]):
                print(f"\t{line}")
    
    return

def extract_figures(doc):

    wrapper = wrapper = textwrap.TextWrapper(width=70)
    
    for i,item in enumerate(doc["figures"]):

        print(f"#/figures/{i}", len(item["captions"]))
        if len(item["captions"])>0:
            for line in wrapper.wrap(text=item["captions"][0]["text"]):
                print(f"\t{line}")
    
    return

def extract_references(doc):

    page_items = doc["page-items"]
    texts = doc["texts"]

    df = pd.DataFrame(doc["instances"]["data"],
                      columns=doc["instances"]["headers"])
    #print(doc["instances"]["headers"])
    
    wrapper = wrapper = textwrap.TextWrapper(width=70)
    
    for i,item in enumerate(doc["texts"]):

        path = f"#/texts/{i}"
        
        labels=[]
        for row in item["properties"]["data"]:
            labels.append(row[item["properties"]["headers"].index("label")])

        if "reference" in labels:

            print(f"text: ") #{type_}, labels: ", ",".join(labels))
            for line in wrapper.wrap(text=item["text"]):
                print(f"\t{line}")            
                
            refs = df[df["subj_path"]==path][["subj_path", "type", "subtype", "original"]]
            print(refs)            
                
    return

if __name__ == '__main__':

    json_files = parse_arguments()

    for json_file in json_files:

        print(f" --> reading {json_file}")
        with open(json_file, "r") as fr:
            doc = json.load(fr)


        extract_text(doc)

        extract_sentences(doc)
        
        extract_tables(doc)
        
        extract_figures(doc)
        
        extract_references(doc)
